#' @title Freundlich Isotherm Non-Linear Analysis
#' @name freundlichanalysis
#' @description This isotherm model is an empirical model applicable to diluted
#' solutions adsorption processes. Furthermore, this model gives an equation which
#' defines the surface heterogeneity and the exponential distribution of active sites.
#' @param Ce the numerical value for the equilibrium capacity
#' @param Qe the numerical value for the adsorbed capacity
#' @import nls2
#' @import Metrics
#' @import stats
#' @import ggplot2
#' @return the nonlinear regression, parameters for Freundlich isotherm,
#' and model error analysis
#' @examples Ce <- c(0.01353, 0.04648, 0.13239, 0.27714, 0.41600, 0.63607, 0.80435, 1.10327, 1.58223)
#' @examples Qe <- c(0.03409, 0.06025, 0.10622, 0.12842, 0.15299, 0.15379, 0.15735, 0.15735, 0.16607)
#' @examples freundlichanalysis(Ce,Qe)
#' @author Jemimah Christine L. Mesias
#' @author Chester C. Deocaris
#' @references Freundlich, H. 1907. Ueber die adsorption in loesungen. Z.
#' Phys. Chem.57:385-470
#' @references Foo, K. Y., &amp; Hameed, B. H. (2009, September 13).
#' <doi:10.1016/j.cej.2009.09.013> Insights into the modeling of adsorption isotherm
#' systems. Chemical Engineering Journal.
#' @export
#'

# Building the Freundlich isotherm nonlinear form
freundlichanalysis <- function(Ce, Qe){

  x <- Ce
  y <- Qe
  data <- data.frame(x, y)

# Freundlich isotherm nonlinear equation
  fit1 <- y ~ (KF * (x^(1/n)))

# Setting of starting values
  start1 <- data.frame(KF = c(0, 100), n = c(0 , 100))

# Fitting of Freundlich isotherm via nls2

  fit2 <- nls2::nls2(fit1, start=start1,  data=data,
                 control= nls.control(maxiter=45, warnOnly=TRUE),
                 algorithm= "port")

  print("Freundlich Isotherm Analysis")
  print(summary(fit2))

  print("Akaike Information Criterion")
  print(AIC(fit2))

  print("Bayesian Information Criterion")
  print(BIC(fit2))

# Error analysis of the Freundlich isotherm model

  errors <- function(y){
    rmse <- Metrics::rmse(y, predict(fit2))
    mae <- Metrics::mae(y, predict(fit2))
    mse <- Metrics::mse(y, predict(fit2))
    rae <- Metrics::rae(y, predict(fit2))
    N <- nrow(na.omit(data))
    SE <- sqrt((sum(y-predict(fit2))^2)/(N-2))
    colnames(y) <- rownames(y) <- colnames(y)
    list("Root Mean Squared Error" = rmse,
           "Mean Absolute Error" = mae,
           "Mean Squared Error" = mse,
           "Root Absolute Error" = rae,
           "Standard Error for the Regression S" = SE)
    }
    a <- errors(y)
    print(a)

# Graphical representation of the Freundlich isotherm model

  ### Predicted parameter values
  parsFreundlich <- as.vector(coefficients(fit2))
  pars_KF <- parsFreundlich[1L];
  pars_n <- parsFreundlich[2L]

  rhs <- function(x){(pars_KF*(x^(1/pars_n)))}

  #### Plot details
  ggplot2::theme_set(ggplot2::theme_bw(10))
  ggplot2::ggplot(data, ggplot2::aes(x = x, y = y)) + ggplot2::geom_point(color ="#3498DB" ) +
    ggplot2::geom_function(color = "#D35400", fun = rhs ) +
    ggplot2::labs(x = "Ce",
         y = "Qe",
         title = "Freundlich Isotherm Nonlinear Model",
         caption = "PUPAIM 0.3.0") +
    ggplot2::theme(plot.title=ggplot2::element_text(hjust=0.5))
}



